﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Media.Imaging;
using System.Windows.Media.Media3D;
using System.Windows.Navigation;
using System.Windows.Shapes;

namespace Praktikum_3_Loesung
{
    public partial class MainWindow : Window
    {
        // Die Kamera steht hier, damit wir sie
        // später verschieben können.
        // Dies benötigt "using System.Windows.Media.Media3D;"
        // (Glühlampen-Menü!)
        PerspectiveCamera kamera = new PerspectiveCamera();

        static Random würfel = new Random();

        public MainWindow()
        {
            InitializeComponent();

            viewport.Camera = kamera; // Zum Grübeln: Warum darf man dies sofort tun, obwohl die Kamera noch nicht eingestellt ist? (Tipp: Unterschied zwischen class und struct)
            kamera.NearPlaneDistance = 10.0;
            kamera.FarPlaneDistance = 20000.0;
            kamera.Position = new Point3D(0.0, 2.0, 2000.0);
            kamera.UpDirection = new Vector3D(0.0, 1.0, 0.0);
            kamera.LookDirection = new Vector3D(0.0, 1.0, -5.0);
            kamera.FieldOfView = 50.0;

            ModelVisual3D visual = new ModelVisual3D();
            viewport.Children.Add(visual);

            Model3DGroup gruppe = new Model3DGroup();
            visual.Content = gruppe;

            DirectionalLight licht = new DirectionalLight();
            gruppe.Children.Add(licht);
            licht.Color = Colors.White;
            licht.Direction = new Vector3D(2.0, -3.0, -2.0);

            for (int i = 0; i < 5; i++)
            {
                for (int j = 0; j < 5; j++)
                {
                    gruppe.Children.Add(ErzeugeTurbine(300.0 * (i - 2), 300.0 * (j - 2), 100.0, 100.0));
                }
            }
        }

        MeshGeometry3D ErzeugeRöhre(double länge, double radius)
        {
            const int abschnitteAxial = 50;
            const int abschnitteTangential = 20;
            MeshGeometry3D mesh = new MeshGeometry3D();

            //// zum Testen vielleicht zunächst nur ein Rechteck:
            //mesh.Positions.Add(new Point3D(0, -radius, 0));
            //mesh.Positions.Add(new Point3D(länge, -radius, 0));
            //mesh.Positions.Add(new Point3D(länge, radius, 0));
            //mesh.Positions.Add(new Point3D(0, radius, 0));
            //mesh.Normals.Add(new Vector3D(0, 0, 1));
            //mesh.Normals.Add(new Vector3D(0, 0, 1));
            //mesh.Normals.Add(new Vector3D(0, 0, 1));
            //mesh.Normals.Add(new Vector3D(0, 0, 1));
            //mesh.TriangleIndices.Add(0);
            //mesh.TriangleIndices.Add(1);
            //mesh.TriangleIndices.Add(2);
            //mesh.TriangleIndices.Add(0);
            //mesh.TriangleIndices.Add(2);
            //mesh.TriangleIndices.Add(3);

            // wirklich "<="
            for (int i = 0; i <= abschnitteAxial; i++)
            {
                double x = länge * i / abschnitteAxial;

                for (int j = 0; j < abschnitteTangential; j++)
                {
                    double phase = 2.0 * Math.PI * j / abschnitteTangential;
                    double yn = Math.Cos(phase);
                    double zn = Math.Sin(phase);
                    double y = radius * yn;
                    double z = radius * zn;

                    mesh.Positions.Add(new Point3D(x, y, z));
                    mesh.Normals.Add(new Vector3D(0.0, yn, zn));

                    if (i < abschnitteAxial)
                    {
                        int i1 = i + 1;
                        int j1 = (j + 1) % abschnitteTangential;
                        mesh.TriangleIndices.Add(j + i * abschnitteTangential);
                        mesh.TriangleIndices.Add(j1 + i1 * abschnitteTangential);
                        mesh.TriangleIndices.Add(j + i1 * abschnitteTangential);
                        mesh.TriangleIndices.Add(j + i * abschnitteTangential);
                        mesh.TriangleIndices.Add(j1 + i * abschnitteTangential);
                        mesh.TriangleIndices.Add(j1 + i1 * abschnitteTangential);
                    }
                }
            }

            return mesh;
        }

        Model3DGroup ErzeugeTurbine(double x, double z, double nabenhöhe, double rotordurchmesser)
        {
            Model3DGroup turbine = new Model3DGroup();
            turbine.Transform = new TranslateTransform3D(x, 0.0, z);

            DiffuseMaterial material = new DiffuseMaterial();
            material.Brush = Brushes.White;

            GeometryModel3D turm = new GeometryModel3D();
            turbine.Children.Add(turm);
            turm.Geometry = ErzeugeRöhre(nabenhöhe, 4.0);
            turm.Material = material;
            turm.Transform = new RotateTransform3D(new AxisAngleRotation3D(new Vector3D(0.0, 0.0, 1.0), 90.0));

            double grundwinkel = 360.0 * würfel.NextDouble();

            Model3DGroup rotor = new Model3DGroup();
            turbine.Children.Add(rotor);
            Transform3DGroup rotorTransformation = new Transform3DGroup();
            rotor.Transform = rotorTransformation;
            AxisAngleRotation3D drehung = new AxisAngleRotation3D(new Vector3D(0.0, 0.0, 1.0), grundwinkel);
            rotorTransformation.Children.Add(new RotateTransform3D(drehung));
            rotorTransformation.Children.Add(new TranslateTransform3D(0.0, nabenhöhe, 6.0));

            for (int i = 0; i < 3; i++)
            {
                GeometryModel3D rotorblatt = new GeometryModel3D();
                rotor.Children.Add(rotorblatt);
                rotorblatt.Geometry = ErzeugeRöhre(0.5 * rotordurchmesser, 3.0);
                rotorblatt.Material = material;
                rotorblatt.Transform = new RotateTransform3D(new AxisAngleRotation3D(new Vector3D(0.0, 0.0, 1.0), 120.0 * i + grundwinkel));
            }

            // Auf das Folgende kommt man nicht allein,
            // sondern beim ersten Mal nur mit Hilfe von Stack Overflow.
            // Die nächste Zeile benötigt "using System.Windows.Media.Animation;".
            DoubleAnimation animation = new DoubleAnimation();
            animation.From = grundwinkel;
            animation.To = grundwinkel - 360.0;
            animation.Duration = new Duration(TimeSpan.FromSeconds(5.0 + 3.0 * würfel.NextDouble()));
            animation.RepeatBehavior = RepeatBehavior.Forever;
            drehung.BeginAnimation(AxisAngleRotation3D.AngleProperty, animation);

            return turbine;
        }

        private void Window_KeyDown(object sender, KeyEventArgs e)
        {
            const double schrittweite = 10.0;
            Point3D position = kamera.Position;
            
            switch (e.Key)
            {
                case Key.Right:
                    position.X += schrittweite;
                    break;
                case Key.Left:
                    position.X -= schrittweite;
                    break;
                case Key.Up:
                    position.Z += schrittweite;
                    break;
                case Key.Down:
                    position.Z -= schrittweite;
                    break;
            }

            // Zum Grübeln: Warum ist diese Zeile nicht überflüssig? (Tipp: Unterschied zwischen class und struct)
            kamera.Position = position;
        }
    }
}
